/**
 * Created by janis dähne.
 */
import {DoExerciseFromBackend} from "../types/doExercise";
import * as genericLayer from './genericLayer'
import {
  AssessedSubmitTestFromBackend, AssessmentForUserWithTestResults,
  StoredTestResultTupleFromBackend
} from "../types/testResults";
import {SolutionVersionDoExerciseFullBase, SolutionVersionTupleFromBackend} from "../types/exerciseSolution";
import {
  CustomTestForBackend, CustomTestFromBackend, RunnableCustomTestFrontendExclusive,
  UpdateCustomTestIndicesTuple
} from "../types/customTests";


const callingName = 'doExerciseLayer'
const controllerPrefix = 'exercise/do'

/**
 * gets the exercise for the release (this also ensures that the user has a participation & solution & 1 solution versions)
 * @param releaseCode the release code (identifies the release)
 */
export async function getDoExercise(releaseCode: string): Promise<DoExerciseFromBackend> {
  return genericLayer.get<DoExerciseFromBackend>(callingName,
    `${controllerPrefix}/${releaseCode}`)
}

/**
 * returns a solution version for the exercise (identified by the release code & plang)
 * @param releaseCode the exercise release code
 * @param plangId the plang
 * @param solutionVersionId null: get the latest solution version, number: get specific solution version
 * @returns {Promise<SolutionVersionDoExerciseFullBase>}
 */
export async function getSolutionVersion(releaseCode: string, plangId: number,
                                         solutionVersionId?: number): Promise<SolutionVersionTupleFromBackend> {

  return genericLayer.get<SolutionVersionTupleFromBackend>(callingName,
    `${controllerPrefix}/solution/${releaseCode}/${plangId}/${solutionVersionId}`)
}

/**
 * sets should not count to true (exercise release participation --> for all plangs/solutions for this release)
 * @param {string} releaseCode
 * @returns {Promise<void>}
 */
export async function setShouldNotCount(releaseCode: string): Promise<void> {

  return genericLayer.update<{}, void>(callingName,
    `${controllerPrefix}/solution/shouldnotcount/${releaseCode}`, {})
}


/**
 * returns the test results for the normal tests & the custom tests (test results are specific for plang & solution version)
 * @param releaseCode the exercise release code
 * @param plangId the plang
 * @param solutionVersionId the solution version to get the test results for
 * @returns {Promise<StoredTestResultTupleFromBackend>}
 */
export async function getAllTestResults(releaseCode: string, plangId: number,
                                        solutionVersionId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
    `${controllerPrefix}/testresults/${releaseCode}/${plangId}/${solutionVersionId}`)
}


/**
 * tries to return the submit test results and manual assessment (only if the release ended else error)
 * @param releaseCode the release code
 * @param pLangId the plang id
 * @returns {Promise<AssessmentForUserWithTestResults>}
 */
export async function getSubmitTestResults(releaseCode: string, pLangId: number): Promise<AssessmentForUserWithTestResults> {
  return genericLayer.get<AssessmentForUserWithTestResults>(callingName,
    `${controllerPrefix}/testresults/submit/${releaseCode}/${pLangId}`)
}


/**
 * saves the given solution version
 *
 * when this method throws (reject) then the error contains the request ... when the request code is
 * InvalidRequest and the data contains a string, then this string a the invalid file name
 * @param releaseCode the exercise release code
 * @param solutionVersion the solution version to save
 */
export async function saveSolutionVersion(releaseCode: string, solutionVersion: SolutionVersionDoExerciseFullBase): Promise<SolutionVersionTupleFromBackend> {

  return genericLayer.update<SolutionVersionDoExerciseFullBase, SolutionVersionTupleFromBackend>(callingName,
    `${controllerPrefix}/save/${releaseCode}`, solutionVersion)
}


/**
 * creates a new custom test
 * @param releaseCode the exercise release code
 * @param customTest the custom test to create
 */
export async function createCustomTest(releaseCode: string, customTest: CustomTestForBackend): Promise<CustomTestFromBackend> {

  return genericLayer.create<CustomTestForBackend, CustomTestFromBackend>(callingName,
    `${controllerPrefix}/create/customtest/${releaseCode}`, customTest)
}

/**
 * updates the given custom test
 * @param releaseCode the exercise release code
 * @param customTest the custom test to create
 */
export async function updateCustomTest(releaseCode: string, customTest: CustomTestForBackend): Promise<CustomTestFromBackend> {
  return genericLayer.update<CustomTestForBackend, CustomTestFromBackend>(callingName,
    `${controllerPrefix}/update/customtest/${releaseCode}`, customTest)
}

/**
 * deletes the given custom tests
 * @param releaseCode the exercise release code
 * @param customTest the test to delete
 * @returns {Promise<void>}
 */
export async function deleteCustomTest(releaseCode: string, customTest: CustomTestForBackend | CustomTestFromBackend): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/customtest/${releaseCode}/${customTest.id}`)
}


/**
 * used to update the custom test display indices
 * @param releaseCodes the exercise release code
 * @param customTests the custom tests
 * @returns {Promise<void>}
 */
export async function updateCustomTestDisplayIndices(releaseCodes: string, customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>): Promise<void> {

  const tuples: ReadonlyArray<UpdateCustomTestIndicesTuple> = customTests.map(p => {
    return {
      id: p.id,
      newDisplayIndex: p.displayIndex
    }
  })

  return genericLayer.update<{ readonly tuples: ReadonlyArray<UpdateCustomTestIndicesTuple> }, void>(callingName,
    controllerPrefix + '/update/indices/customtest/' + releaseCodes, {tuples: tuples})
}