/**
 * Created by janis dähne.
 */


import {RequestApiPrefix} from "./constants";
import {ServerResponseWithData} from "../types/response";
import {ResponseCodes} from "../helpers/responseCodes";
import logger, {SeverityLevel} from "../helpers/logger";
import axios from 'axios'
import {callingName as testingLayerCallerName} from './testingLayer'


/**
 * a generic get function
 * @param callerName the name of the caller
 * @param apiPathWithoutPrefix the api path the api path to call ... ApiPrefix will be prefixed
 * @returns {Promise<Tout>} rejects the plain axios response on error
 * @private
 */
export async function get<Tout>(callerName: string, apiPathWithoutPrefix: string): Promise<Tout> {

  const funcName = '[get]'
  const callerNameWrapped = `[${callerName}]`

  return new Promise<Tout>((resolve, reject) => {
    axios.get(RequestApiPrefix + apiPathWithoutPrefix)
      .then(res => {

        //xhr gives 2xx status code
        //normally the server answers with 2xx even if the request gives a bad result...
        //this is to known when some connection problems occur

        const data = res.data as ServerResponseWithData<Tout>

        if (!data) {
          reject(res)
          return
        }

        if (data.code !== ResponseCodes.ok) {
          logger.debug(callerName, `${funcName} error on response: ${JSON.stringify(data)}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
          reject(res)
          return
        }

        logger.debug(callerName, `${funcName} finished`, SeverityLevel.info)

        resolve(data.data)

      })
      .catch(reason => {
        logger.debug(callerName, `${funcName} error: ${reason.message}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
        reject(reason)
      })
  })
}

export async function create<Tin, Tout>(callerName: string, apiPathWithoutPrefix: string, postData: Tin): Promise<Tout> {

  const funcName = '[create]'
  const callerNameWrapped = `[${callerName}]`

  return new Promise<Tout>((resolve, reject) => {
    axios.post(RequestApiPrefix + apiPathWithoutPrefix, postData)
      .then(res => {

        //xhr gives 2xx status code
        //normally the server answers with 2xx even if the request gives a bad result...
        //this is to known when some connection problems occur

        const data = res.data as ServerResponseWithData<Tout>

        if (data.code !== ResponseCodes.ok) {
          logger.debug(callerName, `${funcName} error on response: ${JSON.stringify(data)}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
          reject(res)
          return
        }

        logger.debug(callerName, `${funcName} finished`, SeverityLevel.info)

        resolve(data.data)

      })
      .catch(reason => {
        logger.debug(callerName, `${funcName} error: ${reason.message}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
        reject(reason)
      })
  })
}

/**
 * a generic post function
 * @param callerName the name of the caller
 * @param apiPathWithoutPrefix the api path the api path to call ... ApiPrefix will be prefixed
 * @param {Tin} postData the data to post
 * @returns {Promise<Tout>} rejects the plain axios response on error
 */
export async function update<Tin, Tout>(callerName: string, apiPathWithoutPrefix: string, postData: Tin): Promise<Tout> {

  const funcName = '[post]'
  const callerNameWrapped = `[${callerName}]`

  return new Promise<Tout>((resolve, reject) => {
    axios.post(RequestApiPrefix + apiPathWithoutPrefix, postData)
      .then(res => {

        //xhr gives 2xx status code
        //normally the server answers with 2xx even if the request gives a bad result...
        //this is to known when some connection problems occur

        const data = res.data as ServerResponseWithData<Tout>

        if (data.code !== ResponseCodes.ok) {
          logger.debug(callerName, `${funcName} error on response : ${JSON.stringify(data)}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
          reject(res)
          return
        }

        logger.debug(callerName, `${funcName} finished`, SeverityLevel.info)

        if (data.code === ResponseCodes.ok && callerName === testingLayerCallerName) {
          logger.debug(callerName, `${funcName} --TESTING-- REQUEST: ${JSON.stringify(postData)} --- RESPONSE: ${JSON.stringify(data)}`, SeverityLevel.debug)
        }

        resolve(data.data)

      })
      .catch(reason => {
        logger.debug(callerName, `${funcName} error: ${reason.message}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
        reject(reason)
      })
  })
}


/**
 * a generic delete function (delete is an identifier)
 * deletes only works via url, no post data
 * @param {string} callerName
 * @param {string} apiPathWithoutPrefix
 * @returns {Promise<Tout>}
 */
export async function del<Tout>(callerName: string, apiPathWithoutPrefix: string): Promise<Tout> {

  const funcName = '[delete]'
  const callerNameWrapped = `[${callerName}]`

  return new Promise<Tout>((resolve, reject) => {
    axios.delete(RequestApiPrefix + apiPathWithoutPrefix)
      .then(res => {

        //xhr gives 2xx status code
        //normally the server answers with 2xx even if the request gives a bad result...
        //this is to known when some connection problems occur

        const data = res.data as ServerResponseWithData<Tout>

        if (data.code !== ResponseCodes.ok) {
          logger.debug(callerName, `${funcName} error on response: ${JSON.stringify(data)}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
          reject(res)
          return
        }

        logger.debug(callerName, `${funcName} finished`, SeverityLevel.info)

        resolve(data.data)

      })
      .catch(reason => {
        logger.debug(callerName, `${funcName} error: ${reason.message}, path: ${apiPathWithoutPrefix}`, SeverityLevel.error)
        reject(reason)
      })
  })
}
