/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Icon, List, Popup} from "semantic-ui-react";
import {leftIconPopupOffset} from "../../../constants";
import {joinWith} from "../../../helpers/typeHelpers";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import {HelpPopup} from "../../helpers/helpPopup";
import {HasFileChangedMap, updateHasFileChangedMap} from "../../../helpers/editorHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {initial as initialCodeEditorSettings} from "../../../state/reducers/editorSettingsDialog/editorSettingsReducer"

import ExplorerView from '../multiTabEditorsCommon/explorerView'

import ReactResizeDetector from "react-resize-detector";

import {Editor, EditSession, IEditSession, UndoManager, createEditSession} from 'brace';
import AceEditor from 'react-ace';

import {AceEditSession, EditorTab, MultiTabCodeEditorProps, MultiTabEditorState} from "../multiTabCodeEditorProps";
import {CodeEditorSettingsFullBase} from "../../../types/userData";
import {getEditorInternalThemeName} from "../../../helpers/themes";
import 'brace/ext/searchbox';
import throttle from 'lodash-es/throttle'

//const css = require('../styles.styl');

const filterHistory = function (deltas: any): any {
  return deltas.filter(function (d: any): any {
    return d.group !== "fold";
  });
}

//from https://stackoverflow.com/questions/28257566/ace-editor-save-send-session-on-server-via-post
export const convertSession = function (session: IEditSession): AceEditSession {

  // noinspection TsLint
  const temp = {
    selection: (session.selection as any).toJSON(),
    value: session.getValue(),
    history: {
      undoStack: (session as any).$undoManager.$undoStack.map(filterHistory),
      redoStack: (session as any).$undoManager.$redoStack.map(filterHistory)
    },
    scrollTop: session.getScrollTop(),
    scrollLeft: session.getScrollLeft(),
    options: (session as any).getOptions()
  }

  return temp
}

//from https://stackoverflow.com/questions/28257566/ace-editor-save-send-session-on-server-via-post
/**
 * applies the saved session (dataJSON) to the given session)
 * @param {AceAjax.IEditSession} session
 * @param {AceEditSession} dataJSON
 * @returns {AceAjax.IEditSession}
 */
export const applySavedSessionFromJSON = function (session: IEditSession, dataJSON: AceEditSession | null): IEditSession {

  if (!dataJSON) return session

  const sess = (session as any)
  sess.$undoManager.$doc = session; // workaround for a bug in ace
  sess.setOptions(dataJSON.options);

  // console.log(sess.setOptions)
  // console.log(dataJSON.options)


  sess.$undoManager.$undoStack = dataJSON.history.undoStack;
  sess.$undoManager.$redoStack = dataJSON.history.redoStack;
  sess.selection.fromJSON(dataJSON.selection);
  sess.setScrollTop(dataJSON.scrollTop);
  sess.setScrollLeft(dataJSON.scrollLeft);
  return sess;
};


const mapStateToProps = (rootState: RootState, props: MultiTabCodeEditorProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    editorSettings: (rootState.userDataSettingsSate.userData.settings
    && rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      ? rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : initialCodeEditorSettings) as CodeEditorSettingsFullBase
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


interface AceEditSessionManageObj {
  // noinspection TsLint
  [editorTabId: number]: IEditSession
}

interface AceWrapper {
  readonly editor: Editor
}


//only one multi editor is allowed that is rendered (because we use html id as resize anchor and html id needs to be unique)


/**
 *
 * IMPORTANT:
 * OPERATION: set some new files
 *
 * set the forceEditorUpdateSessionDummy from normal to beforeChangingFiles
 *    this will save the last edit operation to the files (because when onChange is called the changes are not yet pushed to the undo stack/manager)
 *
 *    when somehow the active tab changes before forceEditorUpdateSessionDummy is afterChangingFiles then this will also
 *    rebuild the this.editSessions
 *
 * do your stuff
 *
 * set forceEditorUpdateSessionDummy from beforeChangingFiles to afterChangingFiles
 *    this will rebuild the this.editSessions and merge the file session (cursor pos, undo stack...) in this.editSessions
 *
 *
 * set forceEditorUpdateSessionDummy from afterChangingFiles to normal
 *    this tries to set the current selected file id as the active session
 *
 */
class MultiFileAceEditor extends React.Component<Props, any> {


  aceEditorWrapper: AceWrapper

  /**
   *
   */
  editSessions: AceEditSessionManageObj = {}


  initialSession: IEditSession = createEditSession('initial session', `ace/mode/java` as any)
  activeSession: IEditSession = this.initialSession

  /**
   * should be called when the editor gets resized
   * @type {any}
   */
  forceRerenderThrottled: () => void = null


  // clearSessionUndoStack(session: IEditSession): void {
  //   const manager = session.getUndoManager()
  //   manager.reset()
  //   session.setUndoManager(manager)
  // }

  /**
   * creates new sessions for every file
   * @param {ReadonlyArray<EditorTab>} editorTabs the editor tags we can't use this.props because they might be outdated (see componentWillUpdate)
   */
  rebuildSessionStorage(editorTabs: ReadonlyArray<EditorTab>): void {

    this.editSessions = {}

    // noinspection TsLint
    for (let i = 0; i < editorTabs.length; i++) {
      const tab: EditorTab = editorTabs[i]

      const newSession = createEditSession(tab.content, `ace/mode/${tab.language}` as any)
      newSession.setUseWrapMode(this.props.editorSettings.useWrapping)


      this.editSessions[tab.id] = newSession
    }
  }

  /**
   * applies the saved session states (from the tab) to the stored this.editSessions
   * @param {ReadonlyArray<EditorTab>} editorTabs the editor tags we can't use this.props because they might be outdated (see componentWillUpdate)
   */
  applySavedSessionStates(editorTabs: ReadonlyArray<EditorTab>): void {

    // noinspection TsLint
    const addedSession: Array<EditorTab> = []

    // noinspection TsLint
    for (let i = 0; i < editorTabs.length; i++) {
      const tab: EditorTab = editorTabs[i]
      applySavedSessionFromJSON(this.editSessions[tab.id], tab.editSession)

      addedSession.push({
        ...tab,
        editSession: convertSession(this.editSessions[tab.id])
      })
    }

    this.props.onSomeEditorSessionChanged(
      addedSession
    )

  }

  /**
   * apply the settings to all tabs (used when we change the settings)
   * when we reload the page the tabs has no sessions yet so new sessions with editor settings are created
   * and attached to the tabs
   *
   * but when we change the editor settings after the initial tab session set we always take the tab.editSession
   * to overwrite the this.editSessions options/settings
   * @param {ReadonlyArray<EditorTab>} editorTabs
   */
  applyEditorSettingsToTabs(editorTabs: ReadonlyArray<EditorTab>): void {
    // noinspection TsLint
    for (let i = 0; i < editorTabs.length; i++) {
      const tab: EditorTab = editorTabs[i]

      if (tab.editSession) {
        //only apply settings if we are past the initial render
        //todo maybe better use a copy (readonly style)
        tab.editSession['option'+'s'] = {
          ...tab.editSession.options,
          wrap: this.props.editorSettings.useWrapping ? "free" : "off",
          tabSize: this.props.editorSettings.tabSize
        }
      }
    }
  }

  componentWillMount(): void {

    this.applyEditorSettingsToTabs(this.props.editorTabs)
    this.rebuildSessionStorage(this.props.editorTabs)
    this.applySavedSessionStates(this.props.editorTabs)
    // noinspection TsLint
    this.forceRerenderThrottled = throttle(this.forceRerender, 200)

  }

  componentWillUnmount(): void {
    // noinspection TsLint
    const sessions: Array<EditorTab> = []

    // noinspection TsLint
    for (let i = 0; i < this.props.editorTabs.length; i++) {
      const tab: EditorTab = this.props.editorTabs[i]

      sessions.push({
        ...tab,
        editSession: convertSession(this.editSessions[tab.id])
      })
    }

    this.props.onSomeEditorSessionChanged(
      sessions
    )
  }


  /**
   * updates the editor ref and tries to set the active session
   * @param {AceWrapper} wrapper
   */
  updateEditorRef(wrapper: AceWrapper): void {
    this.aceEditorWrapper = wrapper

    if (wrapper && wrapper.editor) {
      //make sure the active session is set
      //normally this is ensured through the active editor id and componentWillUpdate
      //but if no tab is open first componentWillUpdate and then the editor ref is set...
      // (in componentWillUpdate we use the editor ref to set the session)

      this.activeSession = this.editSessions[this.props.selectedTabId]

      wrapper.editor.setSession(this.activeSession)
    }
  }


  trySetActiveSession(selectedTabId: number, editorTabs: ReadonlyArray<EditorTab>): void {

    this.activeSession = this.editSessions[selectedTabId]

    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {

      if (this.activeSession) {
        this.aceEditorWrapper.editor.setSession(this.activeSession)
      } else {


        if (this.props.forceEditorUpdateSessionDummy === MultiTabEditorState.beforeChangingFiles) {
          //rebuild

          this.rebuildSessionStorage(editorTabs)
          this.applySavedSessionStates(this.props.editorTabs)
        }

        this.aceEditorWrapper.editor.setSession(this.initialSession)
      }

    }
  }

  notifySaveCurrentMissingEdit(): void {

    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      this.props.onSomeEditorSessionChanged(
        [
          {
            ...this.getActiveEditorTab(),
            editSession: convertSession(this.editSessions[this.props.selectedTabId])
          }
        ]
      )
    }

  }

  componentWillUpdate(nextProps: MultiTabCodeEditorProps): void {

    //selected tab changed -- change session
    if (this.props.selectedTabId !== nextProps.selectedTabId) {
      //change the session

      if (this.props.forceEditorUpdateSessionDummy === MultiTabEditorState.normal) {
        this.notifySaveCurrentMissingEdit()
      }

      this.trySetActiveSession(nextProps.selectedTabId, nextProps.editorTabs)
    }

    if (this.props.forceEditorUpdateSessionDummy !== nextProps.forceEditorUpdateSessionDummy) {

      if (nextProps.forceEditorUpdateSessionDummy === MultiTabEditorState.beforeChangingFiles) {

        this.notifySaveCurrentMissingEdit()

      }
      else if (nextProps.forceEditorUpdateSessionDummy === MultiTabEditorState.afterChangingFiles) {

        this.rebuildSessionStorage(this.props.editorTabs)
        this.applySavedSessionStates(this.props.editorTabs)

      }
      else if (nextProps.forceEditorUpdateSessionDummy === MultiTabEditorState.normal) {
        this.trySetActiveSession(this.props.selectedTabId, this.props.editorTabs)
      }

    }

  }

  forceRerender(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      //console.log("resized")
      this.aceEditorWrapper.editor.resize(false)
    }
  }

  getActiveEditorTab(): EditorTab | null {

    if (this.props.editorTabs.length === 0) return null
    return this.props.editorTabs.find(p => p.id === this.props.selectedTabId)
  }

  render(): JSX.Element {

    //this sometimes does not re-evaluate
    // noinspection TsLint
    const activeEditorTab: EditorTab = this.props.editorTabs.find(p => p.id === this.props.selectedTabId)
    if (!activeEditorTab && this.props.editorTabs.length > 0) {
      //we don't have any file to display yet...
    }

    //.filter(p => p.displayIndex >= 0)

    return (
      <div className="fh">
        <div style={{height: this.props.height}}>

          <ReactResizeDetector handleHeight handleWidth onResize={(w,h) => {
            if (this.forceRerenderThrottled) {
              this.forceRerenderThrottled()
            }
          }} />

          <div
            className="multi-editor-wrapper fh"
          >

            <div className="editor-side-menu">
              <div className={this.props.isFileExplorerDisplayed ? 'clickable-toggled' : 'clickable'}
                   onClick={() => {
                     this.props.onChangeIsFileExplorerDisplayed(!this.props.isFileExplorerDisplayed)
                   }}
              >
                {
                  !this.props.isFileExplorerDisplayed &&
                  <Icon.Group size="large">
                    <Icon name="copy" className="white-colored"/>
                    {
                      this.props.hasUnsavedChanges &&
                      <Popup
                        trigger={
                          <Icon corner name='exclamation triangle' className="warning-colored"/>
                        }
                        content={translate(TranslateIds.atLeastOneFileHasChanged)}
                        offset={leftIconPopupOffset}
                      />
                    }
                  </Icon.Group>
                }

                {
                  this.props.isFileExplorerDisplayed &&
                  <Icon.Group size="large">
                    <Icon name="copy" className="toggled-colored"/>
                    {
                      this.props.hasUnsavedChanges &&
                      <Popup
                        trigger={
                          <Icon corner name='exclamation triangle' className="warning-colored"/>
                        }
                        content={translate(TranslateIds.atLeastOneFileHasChanged)}
                        offset={leftIconPopupOffset}
                      />
                    }
                  </Icon.Group>
                }

              </div>

            </div>

            {
              //the file explorer
              this.props.isFileExplorerDisplayed &&
              <ExplorerView
                hasFileChangedMap={this.props.hasFileChangedMap}
                onHasUnsavedChangesChanged={(hasChanges: boolean) => {

                  this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                  this.props.onHasUnsavedChangesChanged(hasChanges)
                }}
                editorTabs={this.props.editorTabs}
                selectedTabId={this.props.selectedTabId}
                onOpenTabClicked={this.props.onOpenTabClicked}
                userCanCreateNewFiles={this.props.userCanCreateNewFiles}
                language={this.props.language}
                newFileName={this.props.newFileName}
                onCreateFileClicked={(editorTab: EditorTab) => {

                  this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                  this.props.onCreateFileClicked(editorTab)
                }}
                onNewFileNameChanged={this.props.onNewFileNameChanged}
                isRenamingFile={this.props.isRenamingFile}
                renamingFileName={this.props.renamingFileName}
                onFileRename={(editorTab: EditorTab, newName: string) => {

                  this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                  this.props.onFileRename(editorTab, newName)
                }}
                onRenamingFileNameChanged={this.props.onRenamingFileNameChanged}
                renamingFileId={this.props.renamingFileId}
                onIsRenamingFileChanged={this.props.onIsRenamingFileChanged}
                onRenamingFileIdChanged={this.props.onRenamingFileIdChanged}
                onRemoveFile={this.props.onRemoveFile}
              />
            }

            {
              //displayes the tab headers (line)
            }
            <div id="multiFileEditor"
                 className="multi-editor-wrapper-right">

              <div className="editor-tabs-line">
                {
                  joinWith(
                    this.props.editorTabs.filter(p => p.displayIndex >= 0).map((tab: EditorTab) => {

                      return (
                        <div key={tab.id}
                             className={`editor-tab-line-item ${tab.id === this.props.selectedTabId ? 'active' : ''}`}

                        >
                          {
                            tab.isReadonly &&
                            <div style={{marginRight: '0'}}>
                              <HelpPopup icon="key" defaultText={translate(TranslateIds.readonlyFile_desc)}/>
                            </div>
                          }
                          {

                            this.props.hasFileChangedMap[tab.id.toString()] &&
                            <div className="changed-file-style" style={{marginRight: '0'}}>
                              <HelpPopup icon="circle"
                                         defaultText={translate(TranslateIds.thisFileWasChangedSinceLastSave)}/>
                            </div>
                          }
                          <div className={tab.id === this.props.selectedTabId ? null : 'clickable'}
                               onClick={() => {
                                 this.props.onOpenTabClicked(tab)
                               }}
                          >
                            {
                              tab.fileNameWithExtensions
                            }
                          </div>
                          <div className="clickable"
                               onClick={() => {
                                 this.props.onCloseTabClicked(tab)
                               }}
                          >
                            <Icon name="remove"/>
                          </div>
                        </div>
                      )
                    }), (i) => {
                      return (
                        <SimpleVDivider className="mar-null" key={i}/>
                      )
                    }
                  )
                }

              </div>

              {
                //the editor area
              }
              <div className={this.props.isFileExplorerDisplayed ? 'editor-area' : 'editor-area-expanded'}>
                {
                  (!activeEditorTab && this.props.editorTabs.length > 0) &&
                  <div className="editor-no-file-selected">
                    {
                      translate(TranslateIds.noFileSelected)
                    }
                  </div>
                }
                {
                  (!activeEditorTab && this.props.editorTabs.length === 0) &&
                  <div className="editor-no-files">
                    {
                      translate(TranslateIds.noFileAvailable)
                    }
                  </div>
                }
                {
                  activeEditorTab &&
                  <AceEditor
                    width="100%"
                    height="100%"
                    focus={true}
                    ref={(editor: any) => {
                      if (editor) {
                        this.updateEditorRef(editor)
                      }
                    }}
                    name={this.props.id}
                    mode={activeEditorTab.language}
                    theme={getEditorInternalThemeName(this.props.editorSettings.theme)}
                    onChange={(newValue: string, event: any) => {

                      this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)

                      const activeTab = this.getActiveEditorTab()

                      const session = this.aceEditorWrapper.editor.getSession()

                      this.props.onChange({
                        ...activeTab,
                        content: newValue,
                        editSession: convertSession(session),
                      }, newValue, event)

                      this.props.onHasUnsavedChangesChanged(true)
                    }}
                    value={activeEditorTab.content}
                    editorProps={{$blockScrolling: true}}
                    readOnly={!this.props.editorTabs || this.props.editorTabs.length === 0 || !activeEditorTab || activeEditorTab.isReadonly}

                    fontSize={this.props.editorSettings.fontSize}
                    wrapEnabled={this.props.editorSettings.useWrapping}
                    tabSize={this.props.editorSettings.tabSize}
                    showPrintMargin={false}
                    highlightActiveLine={true}

                  />
                }
              </div>
            </div>

          </div>

        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MultiFileAceEditor)