/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {List, Popup, Icon, Form} from "semantic-ui-react";
import {leftIconPopupOffset} from "../../../constants";
import MaterialInput from '../../material/materialInput'
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";
import {getMax} from "../../../helpers/mathHelper";
import {isNotEmptyOrSpaces, ValidationFunc} from "../../../validation/validations";
import {InputOnChangeEvent, OnKeyPressEvent} from "../../../types/reactEvents";
import {HasFileChangedMap} from "../../../helpers/editorHelper";
import {HelpPopup} from "../../helpers/helpPopup";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {EditorTab} from "../multiTabCodeEditorProps";
import {errorDialog} from "../../../helpers/dialogHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly hasFileChangedMap: HasFileChangedMap

  readonly editorTabs: ReadonlyArray<EditorTab>
  readonly selectedTabId: number

  readonly onHasUnsavedChangesChanged: (hasChanges: boolean) => void

  /**
   * called when a tab should be opened (e.g. through the explorer)
   */
  readonly onOpenTabClicked: (editorTab: EditorTab) => void


  /**
   * true: user can create, rename, delete files,
   * false: not
   */
  readonly userCanCreateNewFiles: boolean

  readonly onNewFileNameChanged: (name: string) => void

  /**
   * the file name if the user creates a file
   */
  readonly newFileName: string

  /**
   * called when the user created a new file (tab)
   */
  readonly onCreateFileClicked: (editorTab: EditorTab) => void


  /**
   * the language for new files
   */
  readonly language: string

  /**
   * true: display file renaming (then renamingFileId must also be set)
   * false: not,
   */
  readonly isRenamingFile: boolean

  readonly onIsRenamingFileChanged: (isRenaming: boolean) => void

  /**
   * the id of the file to rename
   *
   * we could use just one prop but maybe we want to change this in future versions...
   * also all ids are allowed ( < 0 , > 0, and maybe 0) so only NaN would be ok...
   */
  readonly renamingFileId: number

  readonly onRenamingFileIdChanged: (fileId: number) => void

  /**
   * the new file name for binding to the input
   */
  readonly renamingFileName: string

  /**
   * called when renamingFileName should change
   */
  readonly onRenamingFileNameChanged: (newName: string) => void

  /**
   * called when a file gets renamed
   */
  readonly onFileRename: (editorTab: EditorTab, newName: string) => void

  readonly onRemoveFile: (editorTab: EditorTab) => void

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const validations: ReadonlyArray<ValidationFunc<string>> = [isNotEmptyOrSpaces]

//the key is not important because we only have 1 input
const validationMessageKey: { readonly [key: string]: string } = {
  fileName: 'TODO invalid'
}


class ExplorerView extends React.Component<Props, any> {


  /**
   * checks if the user can create a file from the current state/props
   * @returns {boolean}
   */
  isNewFileNameValid(): boolean {

    if (isNotEmptyOrSpaces(this.props.newFileName)) return true

    return false
  }

  createNewFile(): void {

    // noinspection TsLint
    let newIndex = getMax(this.props.editorTabs, p => p.displayIndex)

    if (newIndex === null) {
      newIndex = 0
    }
    const newFileAsEditorTab: EditorTab = {
      id: getNextFrontendId(),
      fileNameWithExtensions: this.props.newFileName,
      isMainFile: this.props.editorTabs.length === 0,
      displayIndex: newIndex + 1,
      language: this.props.language,
      isReadonly: false,
      content: '',
      editSession: null
    }

    //check if we already have a file with the same name...

    const fileWithSameName = this.props.editorTabs.find(
      (p: EditorTab) => p.fileNameWithExtensions === newFileAsEditorTab.fileNameWithExtensions);

    if (fileWithSameName) {

      errorDialog(translate(TranslateIds.error),
        translate(TranslateIds.couldNotCreateNewFileBecauseFileWithSameNameAlreadyExists))

      return
    }


    this.props.onCreateFileClicked(newFileAsEditorTab)
    this.props.onNewFileNameChanged('')

    this.props.onHasUnsavedChangesChanged(true)

  }

  render(): JSX.Element {
    return (
      <div className="editor-side-panel pad-left-right-half">

        {
          this.props.userCanCreateNewFiles &&
          <div className="flexed">
            <div className={`v-centered ${this.isNewFileNameValid() ? 'clickable' : 'div-disabled'}`}
                 onClick={() => {
                   this.createNewFile()
                 }}
            >
              <Icon name="add"/>
            </div>
            <MaterialInput
              value={this.props.newFileName}
              onChange={(e: InputOnChangeEvent) => {
                this.props.onNewFileNameChanged(e.currentTarget.value)
              }}
              onKeyPress={(e: OnKeyPressEvent) => {

                if (e.key === 'Enter') {
                  this.createNewFile()
                }
              }}
            />
          </div>
        }

        <List className="explorer-list">
          {
            this.props.editorTabs.map(tab => {
              return (
                <div key={tab.id} className="flexed explorer-list-item">
                  {
                    this.props.hasFileChangedMap[tab.id.toString()] &&
                    <div className="changed-file-style">
                      <HelpPopup icon="circle" defaultText={translate(TranslateIds.thisFileWasChangedSinceLastSave)}/>
                    </div>
                  }
                  {
                    tab.isMainFile &&
                    <HelpPopup defaultText={translate(TranslateIds.thisIsTheMainFile)}/>
                  }

                  {
                    //changing state
                  }
                  {
                    (this.props.isRenamingFile && this.props.renamingFileId === tab.id) &&
                    <div className="flexed-no-wrap">
                      <div className="clickable v-centered"
                           onClick={() => {
                             this.props.onFileRename(tab, this.props.renamingFileName)
                             this.props.onIsRenamingFileChanged(false)
                             this.props.onRenamingFileIdChanged(NaN)
                           }}
                      >
                        <Icon name='checkmark'/>
                      </div>

                      <MaterialInput
                        value={this.props.renamingFileName}
                        onChange={(e: InputOnChangeEvent) => {
                          this.props.onRenamingFileNameChanged(e.currentTarget.value)
                        }}
                        onKeyPress={(e: OnKeyPressEvent) => {

                          if (e.key === 'Enter') {
                            //same as accept
                            this.props.onFileRename(tab, this.props.renamingFileName)
                            this.props.onIsRenamingFileChanged(false)
                            this.props.onRenamingFileIdChanged(NaN)
                          }

                          //same as cancel not working for key press, only key down + up
                          // if (e.key === 'Esc') {
                          //   this.props.onIsRenamingFileChanged(false)
                          //   this.props.onRenamingFileIdChanged(NaN)
                          // }

                        }}
                      />

                      <div className="clickable v-centered"
                           onClick={() => {
                             this.props.onIsRenamingFileChanged(false)
                             this.props.onRenamingFileIdChanged(NaN)
                           }}
                      >
                        <Icon name='cancel'/>
                      </div>

                    </div>
                  }


                  {
                    //normal state
                  }
                  {
                    (!this.props.isRenamingFile || this.props.renamingFileId !== tab.id) &&
                    <div className="flexed-no-wrap">

                      {
                        //cannot remove main file...
                        (this.props.userCanCreateNewFiles && !tab.isMainFile ) &&
                        <div className="clickable"
                             onClick={() => {
                               this.props.onRemoveFile(tab)
                               this.props.onHasUnsavedChangesChanged(true)
                             }}
                        >
                          <Icon name='trash'/>
                        </div>
                      }

                      {
                        (this.props.userCanCreateNewFiles && !tab.isMainFile ) &&
                        <Popup
                          trigger={
                            <div className="clickable"
                                 onClick={() => {
                                   this.props.onIsRenamingFileChanged(true)
                                   this.props.onRenamingFileIdChanged(tab.id)
                                   this.props.onRenamingFileNameChanged(tab.fileNameWithExtensions)
                                 }}
                            >
                              <Icon name='font'/>
                            </div>
                          }
                          content={
                            <div>
                              <span>{tab.fileNameWithExtensions}</span>
                            </div>
                          }
                          offset={leftIconPopupOffset}
                          wide
                        />
                      }

                      <div
                        className={tab.id === this.props.selectedTabId ? 'explorer-item ' : 'explorer-item clickable'}
                        onClick={() => {
                          this.props.onOpenTabClicked(tab)
                        }}
                      >
                        {
                          tab.fileNameWithExtensions
                        }
                      </div>
                    </div>

                  }

                </div>
              )
            })
          }
        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ExplorerView)