/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";

import {Button, Checkbox, Form, Header, Icon, InputOnChangeData} from 'semantic-ui-react'
import {returntypeof} from "react-redux-typescript";
import {RootState} from "../../state/reducers/index";
import {bindActionCreators, Dispatch} from "redux";
import MaterialInput from '../material/materialInput'
import {
  setIsRegisterFormDisplayed, setIsRegistering, setRegisterEmail, setRegisterFirstName, setRegisterLastName,
  setRegisterPw,
  setRegisterPwAgain,
  setRegisterToken,
  reset as resetLoginData
} from "../../state/actions/loginActions";
import {InputOnChangeEvent} from "../../types/reactEvents";
import {notifyError} from "../../helpers/notificationHelper";
import Spinner from "../helpers/Spinner";
import * as loginLayer from '../../communicationLayer/loginLayer'
import {askDialog, infoDialog} from "../../helpers/dialogHelper";
import * as cryptHelper from '../../helpers/cryptoHelper'
import {formSpinnerOpacity} from "./loginForm";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";

export interface MyProps {

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {

    isRegistering: rootState.loginDataState.isRegistering,
    registerToken: rootState.loginDataState.registerToken,
    registerPw: rootState.loginDataState.registerPw,
    registerPwAgain: rootState.loginDataState.registerPwAgain,
    registerFirstName: rootState.loginDataState.registerFirstName,
    registerLastName: rootState.loginDataState.registerLastName,
    registerEmail: rootState.loginDataState.registerEmail,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  setIsRegisterFormDisplayed,
  setIsRegistering,
  setRegisterToken,
  setRegisterPw,
  setRegisterPwAgain,
  setRegisterFirstName,
  setRegisterLastName,
  setRegisterEmail,

  resetLoginData,
}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class RegisterForm extends React.Component<Props, any> {


  async registerClicked(): Promise<void> {

    if (this.props.registerPw !== this.props.registerPwAgain) {
      await infoDialog(translate(TranslateIds.password),
        translate(TranslateIds.passwordAndAgainPasswordDontMatch))
      return
    }

    this.props.setIsRegistering(true)

    try {


      await loginLayer.registerUser({
        token: this.props.registerToken,
        firstName: this.props.registerFirstName,
        lastName: this.props.registerLastName,
        email: this.props.registerEmail,
        password: cryptHelper.hash(this.props.registerPw)
      })

      this.props.resetLoginData()

      await infoDialog(translate(TranslateIds.registerSuccessful),
        translate(TranslateIds.registerSuccessful_text))

    } catch (err) {

      //TODO maybe display some more specific errors
      //e.g. something was empty
      //token is already used...
      //see server auth controller > register function
      notifyError(translate(TranslateIds.serverError))
    }
    finally {
      this.props.setIsRegistering(false)
    }

  }

  render(): JSX.Element {
    return (

      <div className="frosted-glass-register fh">
        <div className="frosted-glass-inner absolute-sized-centered-login">

          <div className="corner-div">

            {
              this.props.isRegistering &&
              <Spinner opacity={formSpinnerOpacity}/>
            }

            {
              !this.props.isRegistering &&
              <div className="ui form frosted-glass-form">

                <div className="form-header">
                  <Header as='h1'>
                    {translate(TranslateIds.projectTitle)}  - {translate(TranslateIds.register)}
                  </Header>
                </div>

                <Form.Field required>
                  <label>
                    {translate(TranslateIds.token)}
                  </label>
                  <MaterialInput
                    value={this.props.registerToken}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.setRegisterToken(e.currentTarget.value)
                    }}
                  />
                </Form.Field>

                <Form.Group widths='equal'>
                  <Form.Field required>
                    <label>
                      {
                        translate(TranslateIds.password)
                      }
                    </label>
                    <MaterialInput type="password"
                                   value={this.props.registerPw}
                                   onChange={(e: InputOnChangeEvent) => {
                                     this.props.setRegisterPw(e.currentTarget.value)
                                   }}
                    />
                  </Form.Field>

                  <Form.Field required>
                    <label>
                      {
                        translate(TranslateIds.passwordAgain)
                      }
                    </label>
                    <MaterialInput type="password"
                                   value={this.props.registerPwAgain}
                                   onChange={(e: InputOnChangeEvent) => {
                                     this.props.setRegisterPwAgain(e.currentTarget.value)
                                   }}

                    />
                  </Form.Field>
                </Form.Group>

                <Form.Group widths='equal'>
                  <Form.Field required>
                    <label>
                      {
                        translate(TranslateIds.firstName)
                      }
                    </label>
                    <MaterialInput
                      value={this.props.registerFirstName}
                      onChange={(e: InputOnChangeEvent) => {
                        this.props.setRegisterFirstName(e.currentTarget.value)
                      }}
                    />
                  </Form.Field>

                  <Form.Field required>
                    <label>
                      {
                        translate(TranslateIds.lastName)
                      }
                    </label>
                    <MaterialInput
                      value={this.props.registerLastName}
                      onChange={(e: InputOnChangeEvent) => {
                        this.props.setRegisterLastName(e.currentTarget.value)
                      }}
                    />
                  </Form.Field>
                </Form.Group>

                <Form.Field required>
                  <label>
                    {
                      translate(TranslateIds.email)
                    }
                  </label>
                  <MaterialInput type="email"
                                 value={this.props.registerEmail}
                                 onChange={(e: InputOnChangeEvent) => {
                                   this.props.setRegisterEmail(e.currentTarget.value)
                                 }}
                  />
                </Form.Field>


                <div>
                  <Button
                    onClick={() => {
                      this.props.setIsRegisterFormDisplayed(false)
                    }}
                  >
                    <Icon name="chevron left"/>
                    <span>
                      {
                        translate(TranslateIds.signIn)
                      }
                    </span>

                  </Button>
                  <Button color="blue"
                          onClick={() => {
                            this.registerClicked()
                          }}
                          content={translate(TranslateIds.register)}
                  >
                  </Button>
                </div>

              </div>
            }

          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(RegisterForm)