/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {Input, Icon, InputProps} from "semantic-ui-react";
import {ValidationFunc} from "../../validation/validations";
import {ChangeEvent, CSSProperties, KeyboardEventHandler, SyntheticEvent, TextareaHTMLAttributes} from "react";

//const css = require('./styles.styl');

export interface MyProps {
  readonly placeholder?: string

  readonly value?: string
  readonly onChange?: (event: ChangeEvent<HTMLTextAreaElement>, value: string) => void

  readonly className?: string
  readonly style?: CSSProperties
  /**
   * usage: <MaterialInput disabled>
   * https://stackoverflow.com/questions/37828543/how-to-pass-props-without-value-to-component
   */
  readonly disabled?: boolean

  /**
   * allows scrolling
   */
  readonly readonly?: boolean

  /**
   * only visually hidden (display: hidden)
   */
  readonly isHidden?: boolean

  /**
   * the validations for the input
   *
   *  usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   *  validationRules must be specified in the reducer!!
   */
  readonly validations?: ReadonlyArray<ValidationFunc<any>>

  /**
   * the key of the message when the validation fails
   *
   *
   * usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   * validationMessageKeys must be specified in the reducer!!
   */
  readonly validationMessageKey?: string

  readonly rows?: number

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    placeholder: props.placeholder,
    value: props.value,
    onChange: props.onChange,
    className: props.className,
    style: props.style,
    disabled: props.disabled,
    isHidden: props.isHidden,
    validations: props.validations,
    validationMessageKey: props.validationMessageKey,
    rows: props.rows,
    readonly: props.readonly
    //inputProps: {...props},
    // placeholder: props.placeholder,
    // icon: props.icon,
    // iconClickHandler: props.iconClickHandler,
    // value: props.value,
    // disabled: props.disabled,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 *
 * a material input component
 * not all original (semantic ui input) props are compatible e.g. label needs to be provided
 *
 * usage:
 * <div>
 *   <label>xxx</label>
 *   <MaterialInput />
 * </div>
 *
 */
class MaterialInput extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className={`material-textarea ${this.props.className}`}
           style={this.props.isHidden === true ? {...this.props.style, visibility: 'hidden'} : {...this.props.style}}>
        {
          //note that we pass all props here with
          //https://zhenyong.github.io/react/docs/transferring-props.html or
          //https://zhenyong.github.io/react/docs/jsx-spread.html
          //so we need to reset the className here else it would override the className set in this code
        }
        <textarea
         readOnly={this.props.readonly}
          placeholder={(this.props.placeholder ? this.props.placeholder : null)}
          onChange={(e: ChangeEvent<HTMLTextAreaElement>) => {
            this.props.onChange(e, e.target.value)
          }}
          disabled={(this.props.disabled)} value={this.props.value ? this.props.value : ''}
          rows={this.props.rows}
        />
        <span
          className={
            (this.props.validations && !this.props.validations.every(
              (validation) => validation(this.props.value))) ? 'bar error-bar' :
              'bar'
          }
        >
          </span>
        {
          this.props.validations &&
          <div className="error-colored brake-words">
            {
              !this.props.validations.every((validation) => validation(this.props.value)) &&
              this.props.validationMessageKey
            }
          </div>
        }

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MaterialInput)