/**
 * Created by janis dähne.
 */

import * as React from "react";
import Logger, {SeverityLevel} from "../../../helpers/logger";
import {isReactElement, joinWith, nameOf, getPropertyName} from "../../../helpers/typeHelpers";
import TabHeader from "./tabHeader";
import TabContentHost from "./tabContentHost";
import PanelHeaderControls from "./panelHeaderControls";
import {Icon, Divider} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import {SimpleDivider} from "../../helpers/simpleDivider";
import TabHeaderBar from "./tabHeaderBar";
import {RootState} from "../../../state/reducers/index";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from "react-redux-typescript";
import {connect} from "react-redux";
import {PanelViewSize} from "../panelGrid/panelGrid";

//require('./styles.styl');


interface MyProps {
  readonly title?: string
  readonly icon?: string
  readonly footerIcon?: string
  readonly footerText?: string
  /**
   * true: show maximize, minimize, ...
   * false: not
   */
  readonly showPanelActions?: boolean

  readonly selectedTabIndex: number
  readonly width?: string
  readonly height?: string

  readonly onSelectedTabChanged: (index: number) => void

  /**
   * true: apply the 0.5em padding for the whole content
   *
   * default to true (or undefined)
   */
  readonly applyDefaultPadding?: boolean


  /**
   * the panel view size (normal, vertical, horizontal, full sized) only css stuff
   */
  readonly panelViewSize: PanelViewSize
  readonly onSizeChangedClicked: (size: PanelViewSize) => void

}


const defaultHeight = '100%'
const defaultWidth = 'auto'


const mapStateToProps = (rootState: RootState, props: MyProps) => {

  return {
    title: props.title,
    icon: props.icon,
    footerIcon: props.footerIcon,
    footerText: props.footerText,
    showPanelActions: props.showPanelActions,
    selectedTabIndex: props.selectedTabIndex,
    width: props.width,
    height: props.height,
    onSelectedTabChanged: props.onSelectedTabChanged,
    applyDefaultPadding: (props.applyDefaultPadding === undefined || props.applyDefaultPadding === true) ? true : false,
    panelViewSize: props.panelViewSize,
    onSizeChangedClicked: props.onSizeChangedClicked
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//a panel hosts multiple tabs and has some basic functionality (minimize, maximize, ...)
class Panel extends React.Component<Props, any> {

  componentWillMount(): void {
    //check if we only use proper child elements, this ensures header works
    React.Children.forEach(this.props.children, (child, index) => {
      if (isReactElement(child)) {

        if ((child && child.type === TabContentHost) || child === null) {
          //ok
        } else {
          Logger.error(`a panel can only contain ${nameOf({TabContentHost})} elements`)
        }
      } else {

        //react text element...
        Logger.error(`not a react element | a ${nameOf({Panel})} can only contain ${nameOf({TabContentHost})} elements`)
      }
    })

  }

  render(): JSX.Element {

    return (
      <div className="panel" style={{
        width: this.props.width ? this.props.width : defaultWidth,
        height: this.props.height ? this.props.height : defaultHeight,
        padding: this.props.applyDefaultPadding ? null : '0px'
      }}>

        {
          //re apply padding
        }
        <div
          className={`panel-header-container ${this.props.applyDefaultPadding ? null : 'panel-padding-left-right-top'}`}>


          {
            (this.props.icon || this.props.title) &&
            <div className="panel-header-caption mar-right flexed">
              {this.props.icon &&
              <i className={this.props.icon + ' icon'}/>
              }
              {this.props.title &&
              <div>{this.props.title}</div>
              }
            </div>
          }


          <div className="panel-tab-headers-container mar-right">
            <TabHeaderBar>
              {
                //add a header for every child tab and a separator
                (React.Children && React.Children.count(this.props.children) > 0) &&
                joinWith(React.Children.map(this.props.children, (child: any, index) => {
                    return (child ?
                        <TabHeader key={index}
                                   icon={(child as TabContentHost).props.icon}
                                   title={(child as TabContentHost).props.title}
                                   isActive={this.props.selectedTabIndex === index}
                                   onClick={() => this.props.onSelectedTabChanged(index)}
                                   defaultHelpText={(child as TabContentHost).props.defaultHelpText}

                        />
                        : null
                    )
                  }
                  ),
                  (i) => <SimpleVDivider style={{marginTop: '0.5em'}} key={i}/>)
              }
            </TabHeaderBar>
          </div>

          {
            (this.props.showPanelActions || (this.props.showPanelActions === undefined)) &&
            <PanelHeaderControls
              panelViewSize={this.props.panelViewSize}
              onSizeChangedClicked={this.props.onSizeChangedClicked}
            />
          }

        </div>

        <Divider className="mar-null"/>


        <div className="tab-host" style={{margin: this.props.applyDefaultPadding ? null : '0'}}>
          {
            React.Children.map(this.props.children, (child: any, index) =>
              index === this.props.selectedTabIndex &&
              <div key={index} className="fh">
                {
                  child
                }
              </div>
            )
          }
        </div>
        {
          (this.props.footerText || this.props.footerIcon) &&
          <div>

            <SimpleDivider/>

            <div className="flexed">
              {
                this.props.footerIcon &&
                <i className={this.props.footerIcon + ' icon'}/>
              }
              {
                this.props.footerText &&
                <div>{this.props.title}</div>
              }

            </div>
          </div>
        }

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Panel)