/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Table, Segment, Icon, Input, Label, Popup, Checkbox} from "semantic-ui-react";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {SortDirection} from "../../../../types/sortDirection";
import {isCurrentSortingColumn} from "../../../../helpers/tablesHelper";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {getDoExerciseSiteToLink} from "../../../../constants";
import {Link} from "react-router-dom";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import Tag from '../../../tagComponents/tag'
import {HelpPopup} from "../../../helpers/helpPopup";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'

//const css = require('../styles.styl');

export interface MyProps {
  //readonly test: string
  readonly exercisePreviews: ReadonlyArray<ExercisePreviewFromBackend>

  /**
   * true: display the releases as they were accessed via code,
   * false: display the releases as they are always accessible/accessed (release is visible to all setting)
   */
  readonly displayAsViaCode: boolean

  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: SortDirection
  readonly setSortByKey: (sortByKey: keyof ExercisePreviewFromBackend, sortDirection: SortDirection) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AllAccessibleExercisesList extends React.Component<Props, any> {


  render(): JSX.Element {

    const list: ReadonlyArray<ExercisePreviewFromBackend> =
      this.props.sortDirection === SortDirection.none
        ? this.props.exercisePreviews
        : orderBy(this.props.exercisePreviews, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <div>

        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.open)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey, 'displayName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.displayName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey, 'shortDescription',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('shortDescription', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.description)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey, 'shouldNotCount',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('shouldNotCount', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup icon="flag outline"
                             defaultText={translate(TranslateIds.shouldNotCount)}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.tags)
                  }
                </span>
              </Table.HeaderCell>
              {
                this.props.displayAsViaCode &&
                <Table.HeaderCell
                  sorted={
                    isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey, 'generatedCode',
                      this.props.sortDirection)
                  }
                >
                  <span className="clickable"
                        onClick={() => {
                          this.props.setSortByKey('generatedCode', this.props.sortDirection + 1)
                        }}
                  >
                    {
                      translate(TranslateIds.releaseCode)
                    }
                  </span>
                </Table.HeaderCell>
              }
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey,
                    'availableWorkingTimeInMinutes',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('availableWorkingTimeInMinutes', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.availableWorkingTimeInMin)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExercisePreviewFromBackend>(this.props.sortByKey, 'id',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('id', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.releaseEndAt_label)
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exercisePreview, index) => {
                //we can't use the id for the key because one exercise can have multiple releases...
                return (
                  <Table.Row key={exercisePreview.generatedCode}>
                    <Table.Cell>
                      <Link to={getDoExerciseSiteToLink(exercisePreview.generatedCode)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </Table.Cell>
                    <Table.Cell>
                      <div className="flexed">
                        <Popup
                          trigger={
                            <div>
                              <Icon name="help circle" className="hoverable mar-right-half"/>
                            </div>
                          }
                          content={exercisePreview.displayName}
                          inverted
                        />
                        <span>{exercisePreview.displayName}</span>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {exercisePreview.shortDescription &&
                      <Popup
                        trigger={<Icon name="help circle" className="hoverable"/>}
                        content={exercisePreview.shortDescription}
                        inverted
                      />
                      }
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        <Checkbox checked={exercisePreview.shouldNotCount} disabled/>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {
                        (exercisePreview.tags && exercisePreview.tags.length > 0) &&
                        <HelpPopup
                          wide="very"
                          defaultText={
                            <div className="exercise-tag-wrapper">
                              {
                                exercisePreview.tags.map((tag, index2) => {
                                  return (
                                    <Tag key={tag.id} tag={tag}/>
                                  )
                                })
                              }
                            </div>
                          }
                        />
                      }
                    </Table.Cell>
                    {
                      this.props.displayAsViaCode &&
                      <Table.Cell>
                        <Icon name="clone" className="clickable mar-right-half"
                              onClick={() => {
                                copyToClipboard(exercisePreview.generatedCode)
                              }}
                        />
                        <span>{exercisePreview.generatedCode}</span>
                      </Table.Cell>
                    }

                    <Table.Cell>
                      {
                        (exercisePreview.hasLimitedWorkingTime ?
                            <span>{exercisePreview.availableWorkingTimeInMinutes}</span> :
                            <span>&infin;</span>
                        )
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Manual ? <span>{translate(TranslateIds.manual)}</span> : null
                      }
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Automatic
                        && exercisePreview.releaseDurationType === ReleaseDurationType.Limited ?
                          <span>{displayDateAndTime(exercisePreview.automaticEndAt, -1)}</span> : null
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>

        </Table>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AllAccessibleExercisesList)