import {ReactChild, ReactElement} from "react";
import head from 'lodash-es/head'

/**
 * Created by janis dähne.
 */


/**
 * checks if a given child is a react element (not a ReactText)
 * NULL is also valid e.g. conditional rendering
 * @param child the child
 * @returns {boolean} true: is a react element, false: not (probably a ReactText)
 */
export function isReactElement<T>(child: ReactChild): child is ReactElement<T> {
  return (!(typeof child === 'string' || typeof child === 'number') && !!child) || child === null
}


/**
 * returns the name of a class, usage: nameOf({ClassName})
 * becomes "ClassName"
 *
 * from https://github.com/Microsoft/TypeScript/issues/1579#issuecomment-306758544
 * @param descriptor
 * @returns {P}
 */
export function nameOf<T, P extends keyof T>(descriptor: {[P in keyof T]?: T[P]; }): P {

  const keys = Object.keys(descriptor)

  return head(
    keys
      .filter(p => descriptor.hasOwnProperty(p))
  ) as P
}

/**
 * function to return the name that is a key of the type T (so only prop names that are part of T are allowed)
 *
 * from https://github.com/Microsoft/TypeScript/issues/1579#issuecomment-306765218
 * @param name
 * @returns {keyof T}
 */
export function getPropertyName<T>(name: keyof T): string {
  return name as string;
}

/**
 * basically like the array join function but does not insert strings but elements of type T
 * this is useful if you want to insert separators between elements
 *
 * null (falsy) values are ignored because this is likely from a conditional rendering
 *
 * @param array the array to join
 * @param createElementFunc the creator func to create new instances of type T,
 *  parameter i is an value from 0.. amount created elements
 * @returns {T[]} the array with inserted elements
 * @code
 *
 * joinWith(React.Children.map(this.props.children, (child: any, index) =>
 *  <TabHeader icon={(child as TabContentHost).props.icon}
 *    title={(child as TabContentHost).props.title}
 *    isActive={this.state.selectedTabIndex === index}
 *    onClick={() => this.handleTabHeaderClicked(index)}
 *  />
 *  ),
 *  (i) => <SimpleVDivider key={i}/>)
 *
 */
export function joinWith<T>(array: ReadonlyArray<T>, createElementFunc: (i: number) => T): ReadonlyArray<T> {

  if (!array || array.length === 0) return []

  return array
    .map((val, index) =>
      (index !== array.length - 1 ) && !!val ? [val, createElementFunc(index)] : [val]
    )
    .filter(arr => !!(arr[0]))
    .reduce((arr1, arr2) => arr1.concat(arr2))

}

/**
 * tries to access an object path
 * e.g. one want to access rootState.userDataSettingsSite.systemRole.displayName BUT systemRole is null...
 * usage: tryGet(() => rootState.userDataSettingsSite.systemRole.displayName, '')
 * will return '' in case some property in the chain rootState.userDataSettingsSite.systemRole.displayName is null or undefined
 *
 * TODO transform all when https://github.com/Microsoft/TypeScript/issues/16 is solved
 * @param {() => T} getFunc
 * @param {T} defaultsTo
 * @returns {T}
 */
export function tryGet<T>(getFunc: () => T, defaultsTo: T): T {

  try {
    const res = getFunc()
    return res
  } catch (err) {

  }
  return defaultsTo

}



