/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./editCustomProjectSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {
  CustomProjectDescriptionFullBase,
  CustomProjectFrontendExclusive, CustomProjectSolutionFullBase,
  CustomProjectTestFullBase, RunnableCustomProjectTestFrontendExclusive
} from "../../../types/customProjects";
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";

import {AllActions as GetCustomProjectActions, reducer as getCustomProjectReducer} from './crud/getCustomProjectReducer'
import {
  AllActions as UpdateDescriptionActions,
  reducer as updateCustomProjectDescriptionReducer
} from './crud/updateCustomProjectDescriptionReducer'
import {
  AllActions as UpdateSolutionActions,
  reducer as updateCustomProjectSolutionReducer
} from './crud/updateCustomProjectSolutionReducer'

import {
  AllActions as SolutionEditorActions,
  reducer as solutionEditorReducer,
  State as SolutionEditorState,
  initial as initialSolutiomEditorState
} from './editorReducers/customProjectSolutionEditorReducer'
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {TestAnswerFromBackend} from "../../../types/testsAndTestProtocol";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";

import {
  AllActions as DescriptionActions,
  reducer as customProjectDescriptionReducer,
  initial as initialDescription
} from './subSets/setCustomProjectDescriptionReducer'

import {
  AllActions as SolutionActions,
  reducer as setCustomProjectSolutionReducer,
  initial as initialSolution
} from './subSets/setCustomProjectSolutionReducer'

import {
  AllActions as CreateCustomProjectActions,
  reducer as createCustomProjectReducer
} from './crud/createCustomProjectReducer'
import {
  AllActions as DeleteCustomProjectTestActions,
  reducer as deleteCustomProjectTestReducer
} from './crud/deleteCustomProjectTestReducer'

import {
  AllActions as RunCompileSingleFileActions,
  reducer as runCompileSingleFileCustomProjectReducer
} from './crud/runCompileSingleFileCustomProjectReducer'

import {
  AllActions as RunCustomTestActions,
  reducer as runCustomProjectTestReducer
} from './crud/runCustomProjectTestReducer'

import {
  AllActions as GetTestResults,
  reducer as getCustomProjectTestResultsReducer
} from './crud/getCustomProjectTestResultsReducer'

import {
  AllActions as UpdatePropertiesActions,
  reducer as updateCustomProjectProperties
} from './crud/updateCustomProjectProperties'

import {AllActions as AllJustRunProgramActions, reducer as justRunProgramReducer} from './crud/justRunProgramReducer'
import Logger from "../../../helpers/logger";
import {justRunProgramArgsMaxLength} from "../doExerciseSite/doExerciseSiteReducer";

export type State = {

  readonly isLoading: boolean

  /**
   * solution from backend to determine if we need to save this or not
   */
  readonly initialSolutionFromBackend: CustomProjectSolutionFullBase

  /**
   * description from backend to determine if we need to save this or not
   */
  readonly initialDescriptionFromBackend: CustomProjectDescriptionFullBase

  readonly customProject: CustomProjectFrontendExclusive

  readonly solutionEditorState: SolutionEditorState

  /**
   * set to true after we created a new custom project successfully
   */
  readonly savedNewCustomProject: boolean

  /**
   * the currently selected test to display in the console view
   *
   * no id because it could be RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive
   */
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null

  /**
   * normally the test results are attached to the tests... but for this command we have no test...
   * we only check if the CURRENT (not saved in db) state of the current file compiles
   *
   * this is set via async crud action callback
   */
  readonly lastCompileSingleFileResult: TestAnswerFromBackend | null

  /**
   * same as lastCompileSingleFileResult but for the just run program result
   *
   * this is set via async crud action callback
   */
  readonly lastJustRunProgramResult: TestAnswerFromBackend | null

  /**
   * for the console view
   * true: uses diff instead of real output
   */
  readonly useDiffIfOutputMismatched: boolean

  /**
   * for the console view
   * true: replaces whitespaces with bullets
   */
  readonly useReplaceWhiteSpacesWithBullets: boolean


  readonly topRightSelectedTabIndex: number
  readonly topLeftSelectedTabIndex: number
  readonly bottomRightSelectedTabIndex: number
  readonly bottomLeftSelectedTabIndex: number

  readonly topRightPanelViewSize: PanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize

  readonly hDividerPosOffset: number
  readonly vDividerPosOffset: number

  readonly isChangeTestDialogDisplayed: boolean
  readonly isCustomProjectCodeEditorSettingsDialogDisplayed: boolean

  /**
   * the args for the just run operation
   *
   * must be <= justRunProgramArgsMaxLength
   */
  readonly justRunProgramArgs: string
  readonly isJustRunProgramArgsInputDisplayed: boolean
}

export const initial: State = {

  initialSolutionFromBackend: null,
  initialDescriptionFromBackend: null,
  customProject: {
    id: getNextFrontendId(),
    displayName: '',
    lastEditorPLangId: -1, //set through solution
    description: initialDescription,
    solution: initialSolution,
    tests: []
  },
  solutionEditorState: initialSolutiomEditorState,

  savedNewCustomProject: false,
  isLoading: false,

  selectedConsoleViewTest: null,
  lastCompileSingleFileResult: null,
  lastJustRunProgramResult: null,

  useDiffIfOutputMismatched: false,
  useReplaceWhiteSpacesWithBullets: true,

  topRightSelectedTabIndex: 0,
  topLeftSelectedTabIndex: 0,
  bottomRightSelectedTabIndex: 0,
  bottomLeftSelectedTabIndex: 0,

  topRightPanelViewSize: PanelViewSize.normal,
  topLeftPanelViewSize: PanelViewSize.normal,
  botRightPanelViewSize: PanelViewSize.normal,
  botLeftPanelViewSize: PanelViewSize.normal,

  isChangeTestDialogDisplayed: false,
  isCustomProjectCodeEditorSettingsDialogDisplayed: false,

  hDividerPosOffset: 150,
  vDividerPosOffset: 150,

  justRunProgramArgs: '',
  isJustRunProgramArgsInputDisplayed: false,
}


export interface SET_justRunProgramArgsAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramArgs
  readonly justRunProgramArgs: string
}

export interface SET_isJustRunProgramArgsInputDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isJustRunProgramArgsInputDisplayed
  readonly isJustRunProgramArgsInputDisplayed: boolean
}


export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}

export interface SET_solutionAction extends ActionBase {
  readonly type: ActionType.SET_solution
  readonly solution: CustomProjectSolutionFullBase
}

export interface SET_testsAction extends ActionBase {
  readonly type: ActionType.SET_tests
  readonly tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>
}


//--- console view options

export interface SET_selectedConsoleViewTestAction extends ActionBase {
  readonly type: ActionType.SET_selectedConsoleViewTest
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
    | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive | null
}


export interface SET_useDiffIfOutputMismatchedAction extends ActionBase {
  readonly type: ActionType.SET_useDiffIfOutputMismatched
  readonly useDiffIfOutputMismatched: boolean
}

export interface SET_useReplaceWhiteSpacesWithBulletsAction extends ActionBase {
  readonly type: ActionType.SET_useReplaceWhiteSpacesWithBullets
  readonly useReplaceWhiteSpacesWithBullets: boolean
}

//--- panel selected indices

export interface SET_topRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topRightSelectedTabIndex
  readonly topRightSelectedTabIndex: number
}

export interface SET_topLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topLeftSelectedTabIndex
  readonly topLeftSelectedTabIndex: number
}

export interface SET_bottomRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomRightSelectedTabIndex
  readonly bottomRightSelectedTabIndex: number
}

export interface SET_bottomLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomLeftSelectedTabIndex
  readonly bottomLeftSelectedTabIndex: number
}

// --- panel view sizes

export interface SET_topRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topRightPanelViewSize
  readonly topRightPanelViewSize: PanelViewSize
}

export interface SET_topLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topLeftPanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
}

export interface SET_botRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botRightPanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
}

export interface SET_botLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botLeftPanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize
}

//--- dialogs

export interface SET_isChangeTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeTestDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isCustomProjectCodeEditorSettingsDialogDisplayed
  readonly isDisplayed: boolean
}

//--- panels

export interface SET_hDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_hDividerPosOffset
  readonly hDividerPosOffset: number
}

export interface SET_vDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_vDividerPosOffset
  readonly vDividerPosOffset: number
}


//used to set the initial plang when we create an exercise
export interface SET_initialPLangIdAction extends ActionBase {
  readonly type: ActionType.SET_initialPLangId
  readonly initialPLangId: number
}


//reset test results after saving code
export interface RESET_TestResultsAction extends ActionBase {
  readonly type: ActionType.RESET_TestResults
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | RESET_TestResultsAction
  | SET_displayNameAction
  | SET_solutionAction
  | SET_testsAction

  | SET_selectedConsoleViewTestAction
  | SET_initialPLangIdAction

  | DescriptionActions
  | SolutionActions

  | SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction
  | SET_useDiffIfOutputMismatchedAction
  | SET_useReplaceWhiteSpacesWithBulletsAction

  | SET_hDividerPosOffsetAction
  | SET_vDividerPosOffsetAction

  | SET_topRightSelectedTabIndexAction
  | SET_topLeftSelectedTabIndexAction
  | SET_bottomRightSelectedTabIndexAction
  | SET_bottomLeftSelectedTabIndexAction

  | SET_topRightPanelViewSizeAction
  | SET_topLeftPanelViewSizeAction
  | SET_botRightPanelViewSizeAction
  | SET_botLeftPanelViewSizeAction
  | SET_isChangeTestDialogDisplayedAction

  | SolutionEditorActions

  | GetCustomProjectActions
  | UpdateDescriptionActions
  | UpdateSolutionActions
  | CreateCustomProjectActions
  | DeleteCustomProjectTestActions
  | RunCompileSingleFileActions
  | RunCustomTestActions
  | GetTestResults
  | UpdatePropertiesActions

  | AllJustRunProgramActions

  | SET_justRunProgramArgsAction
  | SET_isJustRunProgramArgsInputDisplayedAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.JUST_RUN_program_PENDING:
    case ActionType.JUST_RUN_program_FULFILLED:
    case ActionType.JUST_RUN_program_REJECTED:
      return justRunProgramReducer(state, action)


    case ActionType.UPDATE_customProjectProperties_PENDING:
    case ActionType.UPDATE_customProjectProperties_FULFILLED:
    case ActionType.UPDATE_customProjectProperties_REJECTED:
      return updateCustomProjectProperties(state, action)

    case ActionType.GET_customProjectTestResults_PENDING:
    case ActionType.GET_customProjectTestResults_FULFILLED:
    case ActionType.GET_customProjectTestResults_REJECTED:
      return getCustomProjectTestResultsReducer(state, action)

    case ActionType.RUN_customProjectTest_PENDING:
    case ActionType.RUN_customProjectTest_FULFILLED:
    case ActionType.RUN_customProjectTest_REJECTED:
      return runCustomProjectTestReducer(state, action)

    case ActionType.RUN_compileSingleFileCustomProject_PENDING:
    case ActionType.RUN_compileSingleFileCustomProject_FULFILLED:
    case ActionType.RUN_compileSingleFileCustomProject_REJECTED:
      return runCompileSingleFileCustomProjectReducer(state, action)

    case ActionType.DELETE_customProjectTest_PENDING:
    case ActionType.DELETE_customProjectTest_FULFILLED:
    case ActionType.DELETE_customProjectTest_REJECTED:
      return deleteCustomProjectTestReducer(state, action)

    case ActionType.CREATE_customProject_PENDING:
    case ActionType.CREATE_customProject_FULFILLED:
    case ActionType.CREATE_customProject_REJECTED:
      return createCustomProjectReducer(state, action)

    case ActionType.SET_initialPLangId:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          solution: {
            ...state.customProject.solution,
            pLangId: action.initialPLangId
          }
        }
      }

    case ActionType.SET_solutionFiles:
    case ActionType.SET_mainFileId:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          solution: setCustomProjectSolutionReducer(state.customProject.solution, action)
        }
      }

    case ActionType.SET_selectedConsoleViewTest:
      return {
        ...state,
        selectedConsoleViewTest: action.selectedConsoleViewTest
      }

    case ActionType.SET_content:
    case ActionType.SET_assets:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          description: customProjectDescriptionReducer(state.customProject.description, action)
        }
      }

    case ActionType.SET_isCustomProjectCodeEditorSettingsDialogDisplayed:
      return {
        ...state,
        isCustomProjectCodeEditorSettingsDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_selectedTabIdCustomProjectSolutionEditor:
    case ActionType.SET_isFileExplorerDisplayedCustomProjectSolutionEditor:
    case ActionType.SET_newFileNameCustomProjectSolutionEditor:
    case ActionType.SET_renamingFileIdCustomProjectSolutionEditor:
    case ActionType.SET_isRenamingFileCustomProjectSolutionEditor:
    case ActionType.SET_renamingFileNameCustomProjectSolutionEditor:
    case ActionType.SET_hasUnsavedChanges:
    case ActionType.SET_hasFileChangedMap:
      return {
        ...state,
        solutionEditorState: solutionEditorReducer(state.solutionEditorState, action)
      }

    case ActionType.UPDATE_customProjectSolution_PENDING:
    case ActionType.UPDATE_customProjectSolution_FULFILLED:
    case ActionType.UPDATE_customProjectSolution_REJECTED:
      return updateCustomProjectSolutionReducer(state, action)

    case ActionType.GET_customProject_PENDING:
    case ActionType.GET_customProject_FULFILLED:
    case ActionType.GET_customProject_REJECTED:
      return getCustomProjectReducer(state, action)

    case ActionType.UPDATE_customProjectDescription_PENDING:
    case ActionType.UPDATE_customProjectDescription_FULFILLED:
    case ActionType.UPDATE_customProjectDescription_REJECTED:
      return updateCustomProjectDescriptionReducer(state, action)


    case ActionType.SET_displayName:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          displayName: action.displayName
        }
      }

    case ActionType.SET_solution:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          solution: action.solution
        }
      }

    case ActionType.SET_tests:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          tests: action.tests
        }
      }


    case ActionType.SET_topRightSelectedTabIndex:
      return {
        ...state,
        topRightSelectedTabIndex: action.topRightSelectedTabIndex
      }

    case ActionType.SET_topLeftSelectedTabIndex:
      return {
        ...state,
        topLeftSelectedTabIndex: action.topLeftSelectedTabIndex
      }

    case ActionType.SET_bottomRightSelectedTabIndex:
      return {
        ...state,
        bottomRightSelectedTabIndex: action.bottomRightSelectedTabIndex
      }

    case ActionType.SET_bottomLeftSelectedTabIndex:
      return {
        ...state,
        bottomLeftSelectedTabIndex: action.bottomLeftSelectedTabIndex
      }

    case ActionType.SET_isChangeTestDialogDisplayed:
      return {
        ...state,
        isChangeTestDialogDisplayed: action.isDisplayed
      }


    case ActionType.SET_hDividerPosOffset:
      return {
        ...state,
        hDividerPosOffset: action.hDividerPosOffset
      }
    case ActionType.SET_vDividerPosOffset:
      return {
        ...state,
        vDividerPosOffset: action.vDividerPosOffset
      }

    case ActionType.SET_topRightPanelViewSize:
      return {
        ...state,
        topRightPanelViewSize: action.topRightPanelViewSize
      }

    case ActionType.SET_topLeftPanelViewSize:
      return {
        ...state,
        topLeftPanelViewSize: action.topLeftPanelViewSize
      }

    case ActionType.SET_botRightPanelViewSize:
      return {
        ...state,
        botRightPanelViewSize: action.botRightPanelViewSize
      }

    case ActionType.SET_botLeftPanelViewSize:
      return {
        ...state,
        botLeftPanelViewSize: action.botLeftPanelViewSize
      }

    case ActionType.SET_useDiffIfOutputMismatched:
      FrontendSettingsManager.setUseDiffIfOutputMismatched(action.useDiffIfOutputMismatched)
      return {
        ...state,
        useDiffIfOutputMismatched: action.useDiffIfOutputMismatched
      }

    case ActionType.SET_useReplaceWhiteSpacesWithBullets:
      FrontendSettingsManager.setUseReplaceWhiteSpacesWithBullets(action.useReplaceWhiteSpacesWithBullets)
      return {
        ...state,
        useReplaceWhiteSpacesWithBullets: action.useReplaceWhiteSpacesWithBullets
      }

    case ActionType.SET_justRunProgramArgs:

      if (action.justRunProgramArgs.length > justRunProgramArgsMaxLength) {
        Logger.log(`too large just run program args, length: ${action.justRunProgramArgs.length}`)
        return state
      }

      return {
        ...state,
        justRunProgramArgs: action.justRunProgramArgs
      }

    case ActionType.SET_isJustRunProgramArgsInputDisplayed:
      return {
        ...state,
        isJustRunProgramArgsInputDisplayed: action.isJustRunProgramArgsInputDisplayed
      }

    case ActionType.RESET_TestResults:
      return {
        ...state,
        customProject: {
          ...state.customProject,
          tests: state.customProject.tests.map<RunnableCustomProjectTestFrontendExclusive>(p => {
            return {...p, result: null}
          }),
        }
      }


    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

